==> sork-passwd_2.2.orig.tar.gz: sork-passwd-2.2.orig/lib/Driver/smbpasswd.php <==
<?php
/**
 * $Horde: passwd/lib/Driver/smbpasswd.php,v 1.4.2.4 2003/01/29 20:25:06 ericr Exp $
 *
 * The poppassd class attempts to change a user's password on a
 * samba server.
 *
 * @author   Rene Lund Jensen <Rene@lundjensen.net>
 * @package  passwd
 */

class Passwd_Driver_smbpasswd extends Passwd_Driver {

    /** Pointer to the socket connection. */
    var $_fp;

    /** Hash containing connection parameters. */
    var $_params;

    /**
     * Constructs a new poppassd Passwd_Driver object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Passwd_Driver_smbpasswd($params = array())
    {
        $this->_params['host'] = array_key_exists('host', $params)
             ? $params['host']
             : 'localhost';
        $this->_params['program'] = array_key_exists('program', $params)
             ? $params['program']
             : '/usr/bin/smbpasswd';
    }

    /**
     * Connect a pipe to the sambaserver using the smbpasswd program
     *
     * @params string $user     The user to change the password for
     * @params string $tmpfile  The name of a tempfile in which to write output
     * @return                  True on success, fatal error on failure
     */
    function _connect($user, $tmpfile) {
      if (!is_executable($this->_params['program'])) {
        return PEAR::raiseError(_("Password module is not properly configured"));
      }

      $cmd = $this->_params['program'] . " -r " . $this->_params['host'] .
             " -s -U \"$user\" &> $tmpfile";
      $this->_fp = popen($cmd, 'w');
      if ( !$this->_fp) {
        return PEAR::raiseError(_("Could not open pipe to smbpasswd."));
      }
      return true;
    }

    /**
     * Disconnect the pipe
     */
    function _disconnect() {
      if (isset($this->_fp)) {
        pclose($this->_fp);
      }
    }

    /**
     * Send a string to the waiting sambaserver
     *
     * @param  $cmd    The string to send to the server.
     */
    function _sendCommand($cmd) {
      if (fputs($this->_fp, $cmd . "\n") == -1) {
          return PEAR::raiseError(_("Error sending data to smbpasswd."));
      }
      sleep(1);
      return true;
    }

    /**
     * Change the user's password.
     *
     * @param   $username    The user for which to change the password.
     * @param   $oldpassword  The old (current) user password.
     * @param   $newpassword  The new user password to set.
     *
     * @return  boolean    True or false based on success of the change.
     */
    function change_password($username,  $oldpassword, $newpassword) {
      $conf = &$GLOBALS['conf'];

      $res = true;

      // clean up user name in case evil characters are in it
      $user = escapeshellcmd($username);

      $tmpfile = tempnam(@$conf['tmpdir'], "smbpasswd.");
      if ( $tmpfile == false) {
        return PEAR::raiseError(_("tempnam() failed!"));
      }

      $res = $this->_connect($user, $tmpfile);
      if (PEAR::isError($res)) {
        return $res;
      } else {
        $res = file($tmpfile);
        if (count($res) != 0) {
          $res = trim($res[count($res)-1]);
        } else {
          $res = true;
        }
      }

      $res = $this->_sendCommand($oldpassword);
      if (PEAR::isError($res)) {
        return $res;
      }

      $res = $this->_sendCommand($newpassword);
      if (PEAR::isError($res)) {
        return $res;
      }

      $res = $this->_sendCommand($newpassword);
      if (PEAR::isError($res)) {
        return $res;
      }

      $res = $this->_disconnect();
      $res = file($tmpfile);

      if (strstr($res[count($res)-1], "Password changed for user") === false) {
        $res = PEAR::raiseError(strrchr(trim($res[count($res)-2]),":"));
      } else {
        $this->reset_credentials($username, $oldpassword, $newpassword);
        $res = true;
      }

      // Remove the temporary file after use.
      // Hopefully this will be unnessesary with the new versions of PHP
      // When procopen comes into play ;-)
      unlink($tmpfile);

      return $res;
    }
}

==> sork-passwd_2.2.1.orig.tar.gz: sork-passwd-2.2.1/lib/Driver/smbpasswd.php <==
<?php
/**
 * $Horde: passwd/lib/Driver/smbpasswd.php,v 1.4.2.5 2004/03/31 21:52:58 jan Exp $
 *
 * The poppassd class attempts to change a user's password on a
 * samba server.
 *
 * @author   Rene Lund Jensen <Rene@lundjensen.net>
 * @package  passwd
 */

class Passwd_Driver_smbpasswd extends Passwd_Driver {

    /** Pointer to the socket connection. */
    var $_fp;

    /** Hash containing connection parameters. */
    var $_params;

    /**
     * Constructs a new poppassd Passwd_Driver object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Passwd_Driver_smbpasswd($params = array())
    {
        $this->_params['host'] = array_key_exists('host', $params)
             ? $params['host']
             : 'localhost';
        $this->_params['program'] = array_key_exists('program', $params)
             ? $params['program']
             : '/usr/bin/smbpasswd';
    }

    /**
     * Connect a pipe to the sambaserver using the smbpasswd program
     *
     * @params string $user     The user to change the password for
     * @params string $tmpfile  The name of a tempfile in which to write output
     * @return                  True on success, fatal error on failure
     */
    function _connect($user, $tmpfile) {
      if (!is_executable($this->_params['program'])) {
        return PEAR::raiseError(_("Password module is not properly configured"));
      }

      $cmd = $this->_params['program'] . " -r " . $this->_params['host'] .
             " -s -U \"$user\" > $tmpfile 2>&1";
      $this->_fp = popen($cmd, 'w');
      if ( !$this->_fp) {
        return PEAR::raiseError(_("Could not open pipe to smbpasswd."));
      }
      return true;
    }

    /**
     * Disconnect the pipe
     */
    function _disconnect() {
      if (isset($this->_fp)) {
        pclose($this->_fp);
      }
    }

    /**
     * Send a string to the waiting sambaserver
     *
     * @param  $cmd    The string to send to the server.
     */
    function _sendCommand($cmd) {
      if (fputs($this->_fp, $cmd . "\n") == -1) {
          return PEAR::raiseError(_("Error sending data to smbpasswd."));
      }
      sleep(1);
      return true;
    }

    /**
     * Change the user's password.
     *
     * @param   $username    The user for which to change the password.
     * @param   $oldpassword  The old (current) user password.
     * @param   $newpassword  The new user password to set.
     *
     * @return  boolean    True or false based on success of the change.
     */
    function change_password($username,  $oldpassword, $newpassword) {
      $conf = &$GLOBALS['conf'];

      $res = true;

      // clean up user name in case evil characters are in it
      $user = escapeshellcmd($username);

      $tmpfile = tempnam(@$conf['tmpdir'], "smbpasswd.");
      if ( $tmpfile == false) {
        return PEAR::raiseError(_("tempnam() failed!"));
      }

      $res = $this->_connect($user, $tmpfile);
      if (PEAR::isError($res)) {
        return $res;
      } else {
        $res = file($tmpfile);
        if (count($res) != 0) {
          $res = trim($res[count($res)-1]);
        } else {
          $res = true;
        }
      }

      $res = $this->_sendCommand($oldpassword);
      if (PEAR::isError($res)) {
        return $res;
      }

      $res = $this->_sendCommand($newpassword);
      if (PEAR::isError($res)) {
        return $res;
      }

      $res = $this->_sendCommand($newpassword);
      if (PEAR::isError($res)) {
        return $res;
      }

      $res = $this->_disconnect();
      $res = file($tmpfile);

      if (strstr($res[count($res)-1], "Password changed for user") === false) {
        $res = PEAR::raiseError(strrchr(trim($res[count($res)-2]),":"));
      } else {
        $this->reset_credentials($username, $oldpassword, $newpassword);
        $res = true;
      }

      // Remove the temporary file after use.
      // Hopefully this will be unnessesary with the new versions of PHP
      // When procopen comes into play ;-)
      unlink($tmpfile);

      return $res;
    }
}
